import React from 'react';
export interface NodeViewContentProps {
    [key: string]: any;
    as?: React.ElementType;
}
export declare const NodeViewContent: React.FC<NodeViewContentProps>;
//# sourceMappingURL=NodeViewContent.d.ts.map