import React from 'react';
export interface NodeViewWrapperProps {
    [key: string]: any;
    as?: React.ElementType;
}
export declare const NodeViewWrapper: React.FC<NodeViewWrapperProps>;
//# sourceMappingURL=NodeViewWrapper.d.ts.map